library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity memory is
  port (
    buss : in std_logic_vector(15 downto 0);
    ldMAR, ldMDR, memWE, selMDR, clk, reset : in std_logic;
    mdrOut : out std_logic_vector(15 downto 0));
end memory;

architecture Behavioral of memory is
  component mem  is
    port(clk :   in std_logic;
          we   : in std_logic;
          addr : in std_logic_vector;
          din :  in std_logic_vector(15 downto 0);
          dout : out std_logic_vector(15 downto 0));
  end component mem;


  
  signal MAR, memOut, mdrMux, mdr : std_logic_vector(15 downto 0);
    
begin

  mdrout <= mdr;

  -- mdr and mar regs
  process(clk, reset)
  begin
    if (reset = '1') then
      mar <= (others => '0');
      mdr <= (others => '0');
    elsif (clk = '1' and clk'event) then
      -- enable mar
      if (ldMAR = '1') then
        mar <= buss;
      end if;
      -- enable mdr
      if (ldMDR = '1') then
        mdr <= mdrMux;
      end if;
    end if;
  end process;

  -- mdrMUX 
  mdrMux <= buss when selMDR = '0' else
            memOut when selMDR = '1';
  
  mem32_inst : mem
    port map (
      addr => mar(4 downto 0),
      dout => memOut,
      din => mdr,
      we => memWE,
      clk => clk);
  
end behavioral;

