LIBRARY IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE IEEE.std_logic_unsigned.all;

ENTITY tb_memory IS
END tb_memory;

ARCHITECTURE testbench_arch OF tb_memory IS

  component memory
  port(
    buss : in std_logic_vector(15 downto 0);        
    ldMAR, ldMDR, memWE, selMDR, clk, reset : in std_logic;
    mdrOut : out std_logic_vector(15 downto 0));
  end component memory;

  constant PERIOD : time := 5 ns;

  signal w_buss, w_mdrOut : std_logic_vector(15 downto 0);
  signal w_reset, w_selmdr, w_memwe, w_ldmdr, w_ldmar : std_logic;
  signal w_clk : std_logic := '0';

begin
  dut : component memory
    port map(
      clk => w_clk,
      reset => w_reset,
      buss => w_buss,
      ldMAR => w_ldmar,
      ldMDR => w_ldmdr,
      memWE => w_memwe,
      selmdr => w_selmdr,
      mdrout => w_mdrout);

  
  w_clk <= not w_clk after PERIOD / 2 ;
  
  STIMULI:
  process  is
  begin
    w_buss <= x"0000";
    w_ldmar <= '0';
    w_ldmdr <= '0';
    w_selmdr <= '1';
    w_memwe <= '0';
    w_reset <= '0';
    wait for PERIOD;
    w_reset <= '1';
    wait for PERIOD;
    w_reset <= '0';
    wait for PERIOD;
    -- load address 0.
    w_ldmdr <= '1';
    wait for PERIOD;
    w_ldmdr <= '0';
    -- write address 0
    w_buss <= x"0000"; -- address
    w_ldmar <= '1';
    wait for PERIOD;
    w_ldmar <= '0';
    -- put buss in mdr
    w_selmdr <= '0';
    w_buss <= x"A5A5";
    w_ldmdr <= '1';
    wait for PERIOD;
    w_buss <= x"0001";
    w_ldmdr <= '0';
    w_selmdr <= '1';
    w_memwe <= '1';
    -- put 1 in address reg
    w_ldmar <= '1';
    wait for PERIOD;
    -- load address 1.
    w_memwe <= '0';
    w_ldmdr <= '1';
    w_buss <= x"0000";
    w_ldmar <= '1';
    wait for PERIOD;
    w_buss <= x"0002";
    wait for PERIOD;
    w_buss <= x"0003";
    wait for PERIOD;
    w_buss <= x"0004";
    wait for PERIOD;
    w_buss <= x"0005";
    wait for PERIOD;
    w_buss <= x"0006";
    
    wait ;

  end process STIMULI;

end testbench_arch;
